/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: bash_parser_c.c,v 1.1 2006/02/24 17:26:06 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: bash_parser_c.c,v $
 * Revision 1.1  2006/02/24 17:26:06  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>

#include "cuiterm.h"
#include "background.h"
#include "terminal.h"
#include "cuilexelements.h"
#include "support.h"
#include "interface.h"
#include "cuiwindow.h"
#include "bash_parser.h"
#include "bash_parser_c.h"
#include "history.h"

static GNode *
parse_tree_find_producer(GNode *act)
{
	CuiLexElement *lex_element;
	GNode *tmp;
	
	g_return_val_if_fail(act != NULL, NULL);
	
	lex_element = act->data;
	switch (lex_element->type){
		case STRING:
		case OPTION:
		case CONCAT:
			return NULL;
			break;
			
		case COMMAND:
			return act;
			break;
			
		/* The producer can be found at the begin of the pipe. The
		 * serial execution (bash ; operator) is a problematic node
		 * because the output of the two program stored in the same
		 * window. */
		case SERIAL_EXECUTION:
		case PIPE:
			return parse_tree_find_producer(
					g_node_first_child(act));
		
		default:
			g_warning("%s(): Unknown node type in parse tree: %d",
					__func__,
					lex_element->type);
	}
	return NULL;
}


static GNode *
parse_tree_find_first_argument(GNode *act)
{
	CuiLexElement *lex_element;
	GNode *tmp;
	
	if (act == NULL)
		return NULL;
	lex_element = act->data;

	switch (lex_element->type){
		case STRING:
			return act;
			
		case OPTION:
			return NULL;
			
		case CONCAT:
			tmp = parse_tree_find_first_argument(
					g_node_first_child(act));
			if (tmp != NULL)
				return tmp;
			else
				return parse_tree_find_first_argument(
					g_node_nth_child(act, 1));
			break;
			
		case COMMAND:
			return parse_tree_find_first_argument(
					g_node_first_child(act));
			break;
			
		case SERIAL_EXECUTION:
		case PIPE:
			return NULL; 
		
		default:
			g_warning("%s(): Unknown node type in parse tree: %d",
					__func__,
					lex_element->type);
	}
	return NULL;
}

/**
 *
 */
gchar *
parse_tree_get_producer_program(GNode *tree)
{
	GNode *producer = parse_tree_find_producer(tree);
	
	if (producer != NULL){
		return ((CuiLexElement *)producer->data)->text;
	}
	return NULL;
}

gchar *
parse_tree_get_first_argument(GNode *tree)
{
	GNode *argument = parse_tree_find_first_argument(tree);

	if (argument != NULL) {
		return ((CuiLexElement *)argument->data)->text;
	}
	return NULL;
}


/*
 * This function finds the given command node in a parse tree if it is exists.
 * Returns NULL if not.
 */
static GNode * 
node_find_program_by_name(GNode *node,
		const gchar *program_name)
{
	CuiLexElement *lex_element;
	GNode *tmp;
	
	if (node == NULL)
		return NULL;
	
	lex_element = node->data;
	
	switch (lex_element->type){
		case STRING:
		case OPTION:
		case CONCAT:
			return FALSE;
			break;
		case COMMAND:
			if (strcmp(lex_element->text, program_name) == 0)
				return node;
			break;
		case SERIAL_EXECUTION:
		case PIPE:
			tmp = node_find_program_by_name(
					g_node_first_child(node),
					program_name);
			if (tmp != NULL)
				return tmp;
			
			return node_find_program_by_name(
					g_node_nth_child(node, 1),
					program_name);
		default:
			g_warning("%s(): Unknown node type in parse tree: %d",
					__func__,
					lex_element->type);
	}
	return FALSE;
}

static
parse_tree_node_check_option(GNode *node, gchar *option)
{
	CuiLexElement *lex_element;
	
	if (node == NULL)
		return FALSE;
	
	lex_element = node->data;
	switch (lex_element->type){
		case STRING:
			return FALSE;
			break;
		case OPTION:
			return strstr(lex_element->text,
					(const char *)option) != NULL;
			break;
		case LONGOPT:
			return strcmp(lex_element->text,
					(const char *)option) == 0;
			break;
			
		case SERIAL_EXECUTION:
		case PIPE:
		case COMMAND:
			return FALSE;
			break;
			
		case CONCAT:
			if (parse_tree_node_check_option(
					g_node_first_child(node), option))
				return TRUE;
			else
				return parse_tree_node_check_option(
						g_node_nth_child(node, 1),
						option);
			break;
		default:
			g_warning("%s(): Unknown node type in parse tree: %d",
					__func__,
					lex_element->type);
	}
	return FALSE;
}

/**
 * parse_tree_check_option:
 * @node: the root node of the parse tree
 * @program: the name of the program
 * @option: the short or long option
 * @returns: TRUE if the option was used
 *
 * Check if the given program has the option by the parse tree. This function
 * checks only the leftmost instance of the command.
 */
gboolean
parse_tree_check_option(GNode *node,
		const gchar *program,
		const gchar *option)
{
	CuiLexElement 	*lex_element;
	GNode 		*leftmost;
	
	g_return_val_if_fail(option != NULL && program != NULL, FALSE);
	leftmost = node_find_program_by_name(node, program);

	if (leftmost == NULL)
		return FALSE;

	return parse_tree_node_check_option(g_node_first_child(leftmost),
			(gchar *)option);
}


gboolean node_debug_print(GNode *node, gpointer data)
{
	CuiLexElement *lex_element = node->data;
	gint depth = g_node_depth(node);
	gint n;
	gchar *type;
	
	for (n = 0; n < depth; ++n)
		printf("   ");
	
	if (lex_element == NULL)
		printf("This is NULL.");
		
	switch (lex_element->type){
		case STRING:
			type = "STRING ";
			break;
		case OPTION:
			type = "OPTION ";
			break;
		case LONGOPT:
			type = "LONGOPT ";
			break;
		case COMMAND:
			type = "COMMAND";
			break;
		case CONCAT:
			type = "CONCAT";
			break;
		case SERIAL_EXECUTION:
			type = "SERIAL_EXECUTION";
			break;
		case PIPE:
			type = "PIPE";
			break;
		default:
			type = "Unknown";
	}
	printf("type: %s(%d) text: '%s'\n", type, 
			lex_element->type, lex_element->text);
	return FALSE;
}

static gchar *
do_parse_tree_check(GNode *act, gchar *program) {
	CuiLexElement *lex_element;
	gchar *program_name;
	
	if(act == NULL)
		return program;
	
	lex_element = act->data;
#ifdef DEBUG
	g_message("%s(): %d", __func__, lex_element->type);
#endif
	switch (lex_element->type) {
		/*
		 * A string is always fine as parameter?
		 */
		case STRING:
			return program;
		/*
		 * An option is always good?
		 */
		case OPTION:
			return program;
		
		case LONGOPT:
			type_check_long_option(lex_element, program);
			return program;
			
		case COMMAND:
			type_check_command_name(lex_element);
			program = lex_element->text;
			do_parse_tree_check(g_node_first_child(act), program);
			return program;
		/*
		 *
		 */
		case CONCAT: 
			do_parse_tree_check(g_node_first_child(act), program);
			do_parse_tree_check(g_node_nth_child(act, 1), program);
			return program;

		case PIPE:
		case SERIAL_EXECUTION:
			program_name = do_parse_tree_check(
					g_node_first_child(act), program);
			return program_name;
		default:
			g_warning("%s(): Unknown node type in parse tree: %d",
					__func__,
					lex_element->type);
			return NULL;
	}
}

/**
 * parse_tree_check:
 * @tree: a BASH parse tree.
 *
 * Checks the semantic of the BASH parse tree and underlines the mistyped parts
 * in it. This function is running as a separated thread.
 */
gboolean 
parse_tree_check(GNode *tree) 
{
	CuiLexElement *command_lex;

	do_parse_tree_check(tree, NULL);
	return FALSE;
}
