/*
 * Copyright (C) 2004, 2005, 2006 Laszlo Pere
 * $Id: dnd.c,v 1.1 2006/02/24 17:19:56 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: dnd.c,v $
 * Revision 1.1  2006/02/24 17:19:56  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#ifdef ALLDEBUG
#  define DND_DEBUG
#endif

#include <stdlib.h>
#include <gnome.h>
#include "configure.h"
#include "widgets.h"
#include "terminal.h"
#include "background.h"
#include "dnd.h"


static GtkTargetEntry target_table[] = {
	{"text/uri-list", 0, TargetTextUriList},
	{"file/inode", 0, TargetFileForInode},
	{"file/mode", 0, TargetFileForMode},
	{"file/links", 0, TargetFileForLinks},
	{"user/name", 0, TargetUserName},
	{"group/name", 0, TargetGroupName},
	{"file/size", 0, TargetFileForSize},
	{"file/date", 0, TargetFileForDate},
	{"file/name", 0, TargetFileForName},
	NULL
};

static void dnd_debug_print_targets(GList * list)
{
	gint n = 0;

	while (list) {
		char *name =
		    gdk_atom_name(GDK_POINTER_TO_ATOM(list->data));
		g_message("%s(): [%d] %s", __func__, n, name);
		g_free(name);
		list = list->next;
		n++;
	}
}


/**
 * dnd_get_file_target_list:
 * @returns: a #GtkTargetList list for file target sources
 *
 * Returns a #GtkTargetList usefull for DND target for regular files.
 */
GtkTargetList *dnd_get_file_target_list(void)
{
	static GtkTargetList *target_list = NULL;

	if (target_list == NULL)
		target_list = gtk_target_list_new(&target_table[1], 7);
	return target_list;
}

/**
 * dnd_get_dir_target_list:
 * @returns: a #GtkTargetList list for directory target sources
 *
 * Returns a #GtkTargetList usefull for DND target for regular directories.
 */
GtkTargetList *dnd_get_dir_target_list(void)
{
	static GtkTargetList *target_list = NULL;

	if (target_list == NULL)
		target_list = gtk_target_list_new(target_table, 9);
	return target_list;
}

gint dnd_init_destination(GtkWidget * widget)
{
#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif
	g_return_val_if_fail(GTK_IS_WIDGET(widget), -1);

	gtk_drag_dest_set(widget,	    /* The widget.            */
			  0,	            /* Auto action flags.     */
			  target_table,	    /* What types to accept.  */
			  9,	            /* How much dest types.   */
			  GDK_ACTION_COPY); /* Method to accept.      */
}

gint dnd_drag_begin(GdkEventMotion * event, gint button, GtkWidget * widget)
{
	Terminal *t;
	Background *bg;
	GList *list_item;
	CuiWidget *cui_widget;
	CuiWidget *chosen_widget;
	gint chosen_size;
	gint x, y;
	GdkDragContext *drag_context;
	GtkTargetList *target_list;
	TargetType t_type;

#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif

	/*
	 * Lets find the background and the last mouse press coordinates
	 */
	t = g_object_get_data(G_OBJECT(widget), "terminal");
	g_assert(t != NULL);
	bg = t->background;
	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(widget),
					      GTK_TEXT_WINDOW_TEXT,
					      bg->last_mouse_x,
					      bg->last_mouse_y, &x, &y);
	/*
	 * Let's find the dnd source with the smallest area.
	 */
	chosen_widget = NULL;
	list_item = bg->bg_widgets;
	while (list_item != NULL) {
		cui_widget = list_item->data;

		if (!cui_widget_is_mouse_over(cui_widget, x, y))
			goto next_item;

		if (!CUI_IS_RECTANGLE(cui_widget) ||
		    !cui_rectangle_is_dnd_source(CUI_RECTANGLE
						 (cui_widget)))
			goto next_item;

		if (chosen_widget == NULL ||
		    cui_widget_get_size(cui_widget) < chosen_size) {
			chosen_widget = cui_widget;
			chosen_size = cui_widget_get_size(chosen_widget);
		}
	      next_item:
		list_item = list_item->next;
	}
	/*
	 * We need to remember the last dnd source.
	 */
	bg->dnd_source_widget = chosen_widget;
	/*
	 * Let's start the DnD operation. If we found a DnD source we offer that
	 * type first, otherwise we offer something other.
	 */
	if (chosen_widget != NULL) {
		t_type = CUI_RECTANGLE(chosen_widget)->source_type;
		target_list = gtk_target_list_new(&target_table[t_type], 1);
	} else {
		target_list = gtk_target_list_new(&target_table[0], 1);
	}
	drag_context = gtk_drag_begin(widget, target_list,
				      GDK_ACTION_COPY, button, 
				      (GdkEvent *)event);
	if (chosen_widget != NULL)
{
	GdkPixbuf *pixbuf;
	GdkRectangle *area;
	gint x, y;
	
	//g_object_set(G_OBJECT(widget), "use-drag-icons", FALSE, NULL);
		
	area = cui_widget_get_area_window_coords(chosen_widget);
	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(widget),
					      GTK_TEXT_WINDOW_TEXT,
					      area->x, area->y, &x, &y);
	g_message("x=%d, y=%d", x, y);
	pixbuf = gdk_pixbuf_get_from_drawable(
			NULL,
			bg->window,
			NULL,
			area->x, area->y,
			0, 0,
			area->width, area->height);
	gtk_drag_set_icon_pixbuf(drag_context, pixbuf, -30, 0);
	
	switch (t_type) {
		case TargetFileForName:
			gtk_drag_source_set_icon_stock  (widget, GTK_STOCK_DIRECTORY);
			break;
		default:
			gtk_drag_source_set_icon_stock  (widget, GTK_STOCK_NO);
	}
	g_free(area);
}
}


static gchar *dnd_create_file_list(gchar * file_list)
{
	gunichar c;
	GString *str = g_string_new("");

	do {
		if (g_ascii_strncasecmp("file://", file_list, 7) == 0) {
			file_list += 7;
			continue;
		}


		c = g_utf8_get_char(file_list);
		file_list = g_utf8_next_char(file_list);

		if (c == '\n')
			continue;

		if (c == '\r') {
			g_string_append_unichar(str, ' ');
			continue;
		}

		g_string_append_unichar(str, c);
	} while (c != '\0');

	return g_string_free(str, FALSE);
}


void
on_main_widget_drag_data_received(GtkWidget *widget,
				  GdkDragContext *drag_context,
				  gint x, gint y,
				  GtkSelectionData * data,
				  guint info,
				  guint time, gpointer user_data)
{
	Terminal *t = g_object_get_data(G_OBJECT(widget), "terminal");
	Background *bg = t->background;
	CuiRectangle *target;
	CuiRectangle *source;
	gchar *file_list;
	gchar *wd;
	gchar *command;

#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif

	g_assert(t != NULL);

	/*
	 * FIXME: this is not nice...
	 */
	g_signal_stop_emission_by_name(widget, "drag-data-received");

	target = CUI_RECTANGLE(background_get_last_dnd_target(bg));
	source = CUI_RECTANGLE(background_get_last_dnd_target(bg));
	g_signal_emit_by_name(target, "drop-data-received", data, NULL);
	gtk_drag_finish(drag_context, TRUE, FALSE, time);
	
}

void
on_secondary_widget_drag_data_received (GtkWidget       *widget,
                                        GdkDragContext  *drag_context,
                                        gint             x,
                                        gint             y,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data)
{
#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif
	on_main_widget_drag_data_received(widget, drag_context, 
			x, y, data, info, time, user_data);
}



/**
 * on_main_widget_drag_motion:
 * @widget: the #GtkWidget where the motion has been found
 * @drag_context: the drag context of the operation
 * @x: the x coordinate of the mouse
 * @y: the y coordinate of the mouse
 * @time: the time when the motion was detected
 * @user_data: the data given by the signal connection 
 * @returns: TRUE if the motion could be handled (always returns TRUE)
 *
 */
gboolean
on_main_widget_drag_motion(GtkWidget * widget,
			   GdkDragContext * drag_context,
			   gint x, gint y, guint time, gpointer user_data)
{
	Terminal *t = g_object_get_data(G_OBJECT(widget), "terminal");

#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
	dnd_debug_print_targets(drag_context->targets);
#endif
	

	background_dnd_motion_notify(widget, t->background, drag_context, x, y);
	/*
	 * If we not call this function we will never get a motion notify again.
	 * I hate this.
	 */
	gdk_drag_status(drag_context, GDK_ACTION_COPY, time);
	return TRUE;
}


gboolean
on_secondary_widget_drag_motion        (GtkWidget       *widget,
                                        GdkDragContext  *drag_context,
                                        gint             x,
                                        gint             y,
                                        guint            time,
                                        gpointer         user_data)
{
#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif
	return on_main_widget_drag_motion(widget, drag_context, 
			x, y, time, user_data);
}


void
on_main_widget_drag_leave(GtkWidget * widget,
			  GdkDragContext * drag_context,
			  guint time, gpointer user_data)
{
	Terminal *t = g_object_get_data(G_OBJECT(widget), "terminal");
#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif

	background_dnd_motion_notify(widget, t->background, drag_context, 
			-1, -1);
}

void
on_secondary_widget_drag_leave         (GtkWidget       *widget,
                                        GdkDragContext  *drag_context,
                                        guint            time,
                                        gpointer         user_data)
{
#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif
	on_main_widget_drag_leave(widget, drag_context, time, user_data);
}


/**
 * on_main_widget_drag_data_get:
 * @widget: the #GtkTextView widget where the DnD was initiated 
 * @drag_context: the #GdkDragContext we created at the begin
 * @data: the #GtkSelectionData we have to fill
 * @info: the size of the data units
 * @time: what was the time when the data was received
 * @user_data: pointer to the data gave by the signal connection command
 *
 * Called when we initialized a DnD as a source widget and the target is asking
 * the data. There are some problems with this function, namely:
 *
 * (1) We store the DnD source CuiWidget with the main_widget object. This is
 * not a nice solution IMHO.
 * 
 */
void
on_main_widget_drag_data_get(GtkWidget * widget,
			     GdkDragContext * drag_context,
			     GtkSelectionData * data,
			     guint info, guint time, gpointer user_data)
{
	CuiRectangle *cui_widget;
	Terminal *t;

#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif
	t = g_object_get_data(G_OBJECT(widget), "terminal");
	/*
	 * It is possible we drag nothing?
	 */
	cui_widget = CUI_RECTANGLE(
			background_get_last_dnd_source(t->background));
	if (cui_widget == NULL)
		return;
	gtk_selection_data_set(data, data->target, 8,
			       cui_widget->source_data,
			       cui_widget->source_size);
}


void
on_secondary_widget_drag_data_get      (GtkWidget       *widget,
                                        GdkDragContext  *drag_context,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data)
{
#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif
	on_main_widget_drag_data_get(widget, drag_context, data, 
			info, time, user_data);
}


/**
 * on_main_widget_drag_drop:
 *
 * Here we have to decide if the DnD is in the "drop zone".
 */
gboolean
on_main_widget_drag_drop(GtkWidget * widget,
			 GdkDragContext * drag_context,
			 gint x, gint y, guint time, gpointer user_data)
{
	Terminal *t = g_object_get_data(G_OBJECT(widget), "terminal");
	gboolean found;
#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif
	/*
	 * This is not nice to stop the emission of every single signal one by
	 * one, it would be much better to completly remove the existing handler
	 * which will accept every drop operation.
	 */
	g_signal_stop_emission_by_name(widget, "drag-drop");
	/*
	 * If not in drop zone.
	 */
	found =
	    background_dnd_motion_notify(widget, t->background, drag_context, 
			    x, y);
	background_dnd_motion_notify(widget, t->background, drag_context, 
			-1, -1);
	if (!found) {
		//gdk_drop_reply(drag_context, FALSE, time);
		return FALSE;
	}

	gtk_drag_get_data(widget, drag_context,
			  //t->background->match, 
			  GDK_POINTER_TO_ATOM(drag_context->targets->data),
			  time);
	return TRUE;
}


gboolean
on_secondary_widget_drag_drop          (GtkWidget       *widget,
                                        GdkDragContext  *drag_context,
                                        gint             x,
                                        gint             y,
                                        guint            time,
                                        gpointer         user_data)
{
#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif
	return on_main_widget_drag_drop(widget, drag_context,
			 x, y, time, user_data);
}


