/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id:$
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log:$
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>

#include <bonobo.h>
#include <gnome.h>
#include <gconf/gconf-client.h>

#include "support.h"
#include "config.h"
#include "history.h"

#if DEBUG_ALL
#   define DEBUG
#endif

static GList *history_list = NULL;


/*
 * Returns FALSE if the parameter is not an application name supposed to get
 * filenames to edit or view.
 */
static gboolean
application(gchar *program) {
	gint n;
	gchar *not_application[] = {
		"cp", "mv", "ls", "cd", "rm", "getfacl", "setfacl", "scp",
		"chmod", "chown", "du", "locate", "mkdir", "grep", "tree", 
		"stat", "diff", "make", "man", ".", "pushd",  

		"ssh", 
		NULL
	};
	
	for (n = 0; not_application[n] != NULL; ++n)
		if (strcmp(not_application[n], program) == 0)
			return FALSE;
	
	if (strchr(program, '=') != NULL)
		return FALSE;
	
	return TRUE;
}

/*
 * Returns TRUE if the given command name is an internal BASH command.
 */
static gboolean
bash_internal(gchar *program) {
	gint n;
	gchar *bash_internals[] = {
		"source",    "alias",     "bg",        "bind",
		"break",     "builtin",   "cd",        "caller", 
		"command",   "compgen",   "complete",  "continue",
		"declare",   "typeset",   "dirs",      "disown",
		"echo",      "enable",    "eval",      "exec",
		"exit",      "export",    "fc",        "fg",
		"getopts",   "hash",      "help",      "history",
		"jobs",      "kill",      "let",       "local",
		"logout",    "popd",      "printf",    "pushd",
		"pwd",       "read",      "readonly",  "return",
		"set",       "shift",     "shopt",     "suspend",
		"test",      "times",     "trap",      "type",
		"ulimit",    "umask",     "unalias",   "unset",
		"wait",
		NULL
	};
	
	for (n = 0; bash_internals[n] != NULL; ++n)
		if (strcmp(bash_internals[n], program) == 0)
			return TRUE;
	
	return FALSE;
}

gchar *
handle_text_editors(gchar *appname, ApplicationOpcode opcode) {
	gint n, higher;
	static app_freq text_editors[] = {
		{"vi",         0},
		{"vim",        0},
		{"gvim",       0},
		{"emacs",      0},
		{"xemacs",     0}, 
		{"gedit",      0},
		{NULL,         0}
	};

	switch (opcode) {
		case OpTryAdd:
			for (n = 0; text_editors[n].appname != NULL; ++n)
				if (strcmp(text_editors[n].appname, appname) == 0){
					text_editors[n].freq++;
					return NULL;
				}
			break;
		case OpSearch:
			higher = 0;
			for (n = 1; text_editors[n].appname != NULL; ++n)
				if (text_editors[n].freq > text_editors[higher].freq)
					higher = n;
			return text_editors[higher].appname;
			break;
		case OpReset:
			for (n = 1; text_editors[n].appname != NULL; ++n)
				text_editors[n].freq = 0;
			break;
		default:
			g_warning("%s(): Unknown opcode: %d", __func__, opcode);
	}
	
	return NULL;
}

static void
history_line_red(
		file_read_progress *file_progress,
		gchar *line)
{
	gint       argc;
	gchar    **argv;
	GError    *error = NULL;
	gchar     *basename;
	gint       n;
	gchar     *colored_command, tmp;	
	
#ifdef DEBUG
	g_message("%s(): line: '%s'", __func__, line);
#endif
	/*
	 *
	 */
	if (!g_shell_parse_argv(line, &argc, &argv, &error)) {
		g_assert(error != NULL);
		g_warning("%s(): %s", error->message);
		return;
	}
#if 0
	/*
	 * We store the commands so we can use it to visualize the history on
	 * the screen.
	 */
	colored_command = g_strdup_printf(
			"<span color=\"dark green\">%s</span>", argv[0]); 
	n = 1;
	while (n < argc) {
		tmp = colored_command;
		g_message("----------------%s(): '%s'", __func__, argv[n]);
		if (*argv[n] == '-') {
			colored_command = g_strconcat(
					colored_command,
					" <span color=\"brown\">",
					argv[n],
					"</span>",
					NULL);
		}else if (*argv[n] == '<' || *argv[n] == '>') {
			colored_command = g_strconcat(
					colored_command,
					" <span color=\"red\">\\",
					argv[n],
					"</span>",
					NULL);
		} else {
			colored_command = g_strconcat(
					colored_command,
					" ", 
					argv[n],
					NULL);
		}

		++n;
		g_free(tmp);
	}
	g_message("->'%s'", colored_command);
#endif
	colored_command = g_strdup(line);	
	colored_command[strlen(colored_command) - 1] = '\0';
	history_list = g_list_append(history_list, colored_command);
	
	handle_text_editors(argv[0], OpTryAdd);
	
	//g_message("%s(): argc=%d argv[0]: '%s'", __func__, argc, argv[0]);
	if (!application(argv[0]))
		goto finalize;
	
	for (n = 1; n < argc; ++n) {
		if (argv[n][0] != '-'){
			basename = g_path_get_basename(argv[n]);
			//g_message("---->%s '%s'", argv[0], basename);
			g_free(basename);
		}
	}
finalize:
	g_strfreev(argv);
}

static gboolean 
_read_from_history(GIOChannel *source, 
		GIOCondition condition, 
		gpointer data)
{
	gchar     *line;
	gint       length;
	GError    *error = NULL;
	GIOStatus  status;
	file_read_progress *file_progress = (file_read_progress *)data;
#ifdef DEBUG
	g_message("%s(): Start.", __func__);
#endif
	g_assert(file_progress != NULL);
	
	/*
	 * If there is no more data we close the channel.
	 */
	if (condition == G_IO_HUP){
		g_message("end of file");
		//g_io_channel_shutdown(source, TRUE, NULL);
		return FALSE;
	}
	/*
	 * Reading one line from the history file.
	 */
	status = g_io_channel_read_line(source, &line, &length, NULL, &error);
	if (status != G_IO_STATUS_NORMAL || line == NULL) {
		if (error != NULL)
			g_warning("%s(): %s", __func__, error->message);
		gnome_appbar_set_progress_percentage(
				GNOME_APPBAR(file_progress->appbar), 0.0);
		gnome_appbar_pop(GNOME_APPBAR(file_progress->appbar));
#ifdef DEBUG
		g_message("Text Editor: '%s'", handle_text_editors(NULL, OpSearch));
#endif
		return FALSE;
	}

	file_progress->red += length;
	gnome_appbar_set_progress_percentage(
			GNOME_APPBAR(file_progress->appbar),
			(gfloat)file_progress->red / 
			  file_progress->size);
	if (length > 1)
		history_line_red(file_progress, line);

	g_free(line);
	
	return TRUE;
}

/**
 * history_read_bash_history:
 * widget: a #GtkWidget to find the message bar by the "appbar1" name.
 *
 * Reads and processes the BASH history file. This function must be executed as
 * a separated thread.
 */
gpointer
history_read_bash_history(GtkWidget *widget)
{
	GIOChannel *history_channel;
	file_read_progress	*file_progress;
	GtkWidget   *appbar;
	gchar       *filename;
	gchar       *message;
	struct stat  stat_buf;
	gchar       line[255];
	gint        red = 0;
#ifdef DEBUG
	g_message("%s(): Start.", __func__);
#endif
	file_progress = g_new(file_read_progress, 1);

	/* Filename and filesize. */
	filename = g_build_path(G_DIR_SEPARATOR_S, 
			g_getenv("HOME"), ".bash_history", NULL);
	if (g_stat(filename, &stat_buf) != 0){
		g_warning("Unable to stat file '%s'", filename);
		return NULL;
	}
	file_progress->size = stat_buf.st_size;
	file_progress->red = 0;
	
	/* Widgets: message and progress */
	file_progress->appbar = lookup_widget(widget, "appbar1");
	message = g_strdup_printf(_("Reading file '%s'"), filename);
  	gnome_appbar_push(GNOME_APPBAR(file_progress->appbar), message);
	g_free(message);
	
	/* Opening file and creating channel. */
	file_progress->ifile = g_open(filename, O_RDONLY);
	if (file_progress->ifile < 0) {
		g_warning("Unable to open file '%s'", filename);
		return NULL;
	}
	history_channel = g_io_channel_unix_new(file_progress->ifile);
	g_io_add_watch_full(history_channel, 
			G_PRIORITY_DEFAULT_IDLE, 
			G_IO_IN | G_IO_HUP, 
			_read_from_history, (gpointer) file_progress, NULL);
	g_thread_exit(NULL);
	return NULL;
}

/*****************************************************************************
 * Type checking and correcting.                                             *
 *                                                                           *
 *****************************************************************************/
static GTree *programs = NULL;

gboolean
save_program_info(gchar *key, program_info *info, FILE *outfile)
{
	GList *li;
	program_option *option;
	g_message("%s(): starting on '%s'", __func__, key);
	
	g_fprintf(outfile, "[%s]\n", key);
	if (info->info != NULL) 
		g_fprintf(outfile, "sdescr='%s'\n", info->info);
	
	li = info->longopts;
	while (li  != NULL) {
		option = li->data;
		g_fprintf(outfile, "longopt='%s'\n", option->option);
		if (option->description != NULL)
			g_fprintf(outfile, "descr='%s'\n", option->description);
next_item:
		li = li->next;
	}
	
	g_fprintf(outfile, "\n");
	return FALSE;
}

static void 
man_cache_finalize(void)
{
	gchar *filename;
	gchar *backup_filename;
	FILE  *outfile;
	
	g_message("%s(): Start.", __func__);
	if (programs == NULL)
		return;
	
	filename = g_build_path(G_DIR_SEPARATOR_S,
			g_getenv("HOME"), ".cuiterm_man_cache", NULL);
	backup_filename = g_build_path(G_DIR_SEPARATOR_S,
			g_getenv("HOME"), ".cuiterm_man_cache.bak", NULL);
	
	g_rename(filename, backup_filename);
	
	outfile = g_fopen(filename, "w");
	
	g_tree_foreach(programs, save_program_info, outfile);

finalize:
	fclose(outfile);	
	g_remove(backup_filename);
	g_free(filename);
	g_free(backup_filename);
}

static inline void 
man_cache_initialize(void)
{
	static gboolean initialized = FALSE;
	
	if (initialized)
		return;
	initialized = TRUE;
	/*
	 * Creating the tree if it does not exists yet.
	 */
	if (programs == NULL)
		programs = g_tree_new(strcmp);
	/*
	 * This function will save the man cache at exit.
	 */
	g_atexit(man_cache_finalize);
}

gint 
_programs_compare(program_info *a, program_info *b)
{
	gchar *aname, *bname;
	
	g_assert(a != NULL);
	g_assert(b != NULL);
	
	aname = a->name;
	bname = b->name;
	g_message("'%s' ? '%s'", aname, bname);
			
	return g_ascii_strcasecmp(
			((program_info *)a)->name,
			((program_info *)b)->name);
}

TypingCheckResult
program_check_longopt_typing(
		const gchar *name, 
		const gchar *longopt)
{
	GList          *li;
	program_option *option;
	program_info   *info;
	
	g_assert(name != NULL);
	
	if (programs == NULL)
		return TypingNotChecked;
	
	info = g_tree_lookup(programs, name);
	if (info == NULL)
		return TypingNotChecked;

	li = info->longopts;
	while (li != NULL) {
		option = li->data;
		if (strcmp(option->option, longopt) == 0)
			return TypingCorrect; 
next_item:
		li = li->next;
	}
	
	return TypingError;
}

gboolean
program_add_info(
	gchar *name,
	gchar *descr)
{
	gboolean retval = FALSE;
	program_info   *info;
	
	g_assert(name != NULL);
#ifdef DEBUG	
	g_message("%s(): Starting with program: '%s'", __func__, name);
#endif
	man_cache_initialize();
	info = g_tree_lookup(programs, name);
	if (info == NULL) {
		info = g_new(program_info, 1);
		info->name = g_strdup(name);
		info->longopts = NULL;
		info->opts = NULL;
		info->info = g_strdup(descr);
		g_tree_insert(programs, info->name, info);
		retval = TRUE;
	}
	info->info = g_strdup(descr);
	return retval;
}

gboolean
program_add_longopt(
	gchar *name, 
	gchar *longopt)
{
	gboolean retval = FALSE;
	program_info   *info;
	program_option *option;
		
	g_assert(name != NULL);
#ifdef DEBUG	
	g_message("%s(): Starting with program: '%s'", __func__, name);
#endif
	man_cache_initialize();
	/*
	 * Finding or insertint the info.
	 */
	info = g_tree_lookup(programs, name);
	if (info == NULL) {
		info = g_new(program_info, 1);
		info->name = g_strdup(name);
		info->longopts = NULL;
		info->opts = NULL;
		info->info = NULL;
		g_tree_insert(programs, info->name, info);
		retval = TRUE;
	}

	/*
	 * FIXME: we should try to find the option first, so we would not get
	 * duplications.
	 */
	option = g_new(program_option, 1);
	option->option = g_strstrip(g_strdup(longopt));
	option->description = NULL;
	info->longopts = g_list_append(info->longopts, option);
	return retval;
}

/**
 *
 */
gboolean
type_check_command_name(CuiLexElement *lexelement)
{
	gchar *path;
	GtkTextBuffer *text_buffer;
	GtkTextIter begin, end;
#ifdef DEBUG	
	g_message("%s(): Start: '%s'", __func__, lexelement->text);
#endif
	
	text_buffer = gtk_text_mark_get_buffer(lexelement->begin);
	gtk_text_buffer_get_iter_at_mark(text_buffer, 
				&begin, lexelement->begin);
	gtk_text_buffer_get_iter_at_mark(text_buffer, 
				&end, lexelement->end);

	path = g_find_program_in_path(lexelement->text);
	if (path == NULL && !bash_internal(lexelement->text)) {
		gtk_text_buffer_apply_tag_by_name(text_buffer,
				"underline-error",
				&begin, &end);
		return FALSE;
	} else {
		gtk_text_buffer_remove_tag_by_name(text_buffer,
				"underline-error",
				&begin, &end);
		return TRUE;
	}
}


gboolean
type_check_long_option(CuiLexElement *longopt, gchar *name)
{
	GtkTextBuffer *text_buffer;
	GtkTextIter begin, end;
	
	text_buffer = gtk_text_mark_get_buffer(longopt->begin);
	gtk_text_buffer_get_iter_at_mark(text_buffer, 
				&begin, longopt->begin);
	gtk_text_buffer_get_iter_at_mark(text_buffer, 
				&end, longopt->end);

	if (program_check_longopt_typing(name, longopt->text) == TypingError) {
		gtk_text_buffer_apply_tag_by_name(text_buffer,
				"underline-error",
				&begin, &end);
		return FALSE;
	} else {
		gtk_text_buffer_remove_tag_by_name(text_buffer,
				"underline-error",
				&begin, &end);
		return TRUE;
	}
}

/*
 * Functions for the history window.
 */




void
on_history_close_button_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	widget = lookup_widget(GTK_WIDGET(button), "history_window");
	gtk_widget_destroy(widget);
}


void
on_history_treeview_realize            (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkTreeView *tree_view = GTK_TREE_VIEW(widget);
	GtkTreeViewColumn *column;
	GtkCellRenderer   *renderer;
	GtkTreeStore *tree_store;
	GtkTreeIter iter_level0;
	
	gint n;
	GList *actual;
#ifdef DEBUG
	g_message("%s(): Start.", __func__);
#endif
	/*
	 * Tree model
	 */
	tree_store = gtk_tree_store_new(1,
			G_TYPE_STRING    /* The command itself.         */
			);
	n = 1;
	actual = history_list;
	while(actual != NULL) {
#ifdef DEBUG
		g_message("%s(): reading into the store: %d", __func__, n);
#endif
		gtk_tree_store_append(tree_store, &iter_level0, NULL);
		gtk_tree_store_set(tree_store, &iter_level0, 
	  		0, actual->data, -1);
next_item:
		actual = actual->next;
		n++;
	}
	gtk_tree_view_set_model(tree_view, GTK_TREE_MODEL(tree_store));
	
	/*
	 * The columns.
	 */
  	/* Column0: the name */
  	renderer = gtk_cell_renderer_text_new ();
  	column = gtk_tree_view_column_new_with_attributes(
		  _("The command"), 
		  renderer, "text" /*"markup"*/, 0, NULL);
  	gtk_tree_view_append_column(tree_view, column);
	gtk_tree_view_set_rules_hint(tree_view, TRUE);
}


