/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: ls_parser_c.c,v 1.1 2006/02/24 17:17:52 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: ls_parser_c.c,v $
 * Revision 1.1  2006/02/24 17:17:52  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>

#include "cuiterm.h"
#include "configure.h"
#include "support.h"
#include "interface.h"
#include "cuilexelements.h"
#include "background.h"
#include "cuiwindow.h"
#include "ls_parser_c.h"
#include "command_generator.h"


CuiWidget *
cui_entry_new_for_filename(
		CuiWindow   *window,
		Background  *background,
		CuiFilename *file,
		GtkTextMark *begin,
		GtkTextMark *end,
		const char  *text)
{
	CuiWidget *widget;
	widget = cui_entry_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_area_by_marks(widget, begin, end);
	cui_entry_set_default_text(CUI_ENTRY(widget), text);
	cui_widget_append_file_command(widget, FileRename, file);
	cui_widget_append_window_command(widget, WindowRefresh, window);
	if (background_check_stored_command(background, CommandNewFile)){
		if (!strcmp(text, "NewFile"))
			background_store_widget(background, widget);
	}
	else if (background_check_stored_command(background, CommandNewDirectory))
		if (!strcmp(text, "NewDirectory"))
			background_store_widget(background, widget);
	return widget;
}

/**
 * cui_file_menu_new_for_filename:
 * @window: this #CuiWindow will contain the new #CuiFileMenu
 * @background: the @Background for the new menu
 * @file: the #CuiFilename for the menu
 * @begin: the begin mark for the new widgets active area
 * @end: the end mark for the new widgets active area
 * @returns: a new #CuiFileMenu
 *
 * Creates a new #CuiFileMenu for the given begin/end points.
 */
CuiFileMenu *
cui_file_menu_new_for_filename(
		CuiWindow   *window,
		Background  *background,
		CuiFilename *file,
		GtkTextMark *begin,
		GtkTextMark *end)
{
	GtkWidget *appbar;
	CuiWidget *widget;
	
	g_assert(background != NULL && file != NULL);
	g_assert(begin != NULL && end != NULL);
	appbar = lookup_widget(background->widget, "appbar1");

	widget = cui_file_menu_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_marks(widget, begin, end);
	cui_widget_set_tooltip(widget, __func__);
	cui_file_menu_set_filename(CUI_FILE_MENU(widget), file);
	cui_widget_append_window_command(widget, WindowRefresh, window);
}

/**
 * cui_action_area_new_for_filename:
 * @window: the embedded window for this action area
 * @background: The #Background where the #CuiActionArea will appear.
 * @file: The file the area will open.
 * @begin: The starting point of the filename on the user interface.
 * @end: The end point of the filename.
 * @returns: A new #CuiActionArea
 *
 * Creates a new #ActionArea for the filename. This action area is used to open
 * a file in an application or show the contents of a directory. The function
 * registrates the new area in the background, so the return value is not needed
 * to use it.
 *
 */
CuiWidget *
cui_action_area_new_for_filename(
		CuiWindow   *window,
		Background  *background,
		CuiFilename *file,
		GtkTextMark *begin,
		GtkTextMark *end)
{
	GtkWidget *appbar;
	CuiWidget *widget;
	gchar *tooltip;
	CuiFileType filetype;
	gchar *filename;
	GtkTextIter biter, eiter;
	gchar *tag = NULL;
	gchar *active_tag = NULL;
	
	g_assert(background != NULL && file != NULL);
	g_assert(begin != NULL && end != NULL);
	appbar = lookup_widget(background->widget, "appbar1");
		
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_marks(widget, begin, end);
	
	filename = cui_filename_get_filename(file);
#ifdef DEBUG
	g_message("%s(): Creating action area for '%s'", __func__, filename);
#endif
	tooltip = g_strdup_printf(
			_("The '%s' is a %s. Double click to open it."), 
			filename, cui_filename_get_typename(file));
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_append_file_command(widget, FileOpen, file);
	if (!cui_filename_is_dir(file))
		cui_widget_append_window_command(widget, WindowRefresh, window);
	/*
	 * The syntax highlight and the active highlight.
	 */
	gtk_text_buffer_get_iter_at_mark(background->text_buffer, 
			&biter, begin);
	gtk_text_buffer_get_iter_at_mark(background->text_buffer, 
			&eiter, end);
	
	if (cui_filename_is_dir(file)){
		tag = "directory";
		active_tag = "directory_active";
	} else if (cui_filename_is_user_execute(file) ||
			cui_filename_is_group_execute(file) ||
			cui_filename_is_others_execute(file) ){
		tag = "executable";
		active_tag = "executable_active";
	} else if (cui_filename_is_compressed(file)){
		tag = "archive";
		active_tag = "archive_active";
	} else if (cui_filename_is_block_device(file)) {
		tag = "block_device";
		active_tag = "block_device_active";
	} else if (cui_filename_is_character_device(file)) {
		tag = "character_device";
		active_tag = "character_device_active";
	} else { 
		tag = "file";
		active_tag = "file_active";
	}
		
	if (tag != NULL)
		gtk_text_buffer_apply_tag_by_name(background->text_buffer, 
				tag, &biter, &eiter);
	if (active_tag != NULL)
		cui_widget_set_active_style(widget, active_tag);

	g_free(filename);
	return widget;
}


CuiWidget *
cui_action_area_new_for_size(
		CuiWindow   *window,
		Background  *background,
		CuiFilename *file,
		GtkTextMark *begin,
		GtkTextMark *end)
{
	GtkWidget *appbar;
	CuiWidget *widget;
	gchar *tooltip;
	gchar *filename;
	
	g_assert(background != NULL && file != NULL);
	g_assert(begin != NULL && end != NULL);
	appbar = lookup_widget(background->widget, "appbar1");
		
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_marks(widget, begin, end);
	cui_widget_set_active_style(widget, "normal_active");

	filename = cui_filename_get_filename(file);
	if (cui_filename_is_dir(file))
		tooltip = g_strdup_printf(
			_("The size of the directory. "
			  "Double click to show the properties for '%s'."),
					filename);
	else
		tooltip = g_strdup_printf(
			_("The size of the file. "
			  "Double click to show the properties for '%s'."),
					filename);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_append_file_command(widget, FileShowProperties, file);
	//if (!cui_filename_is_dir(file))
	//	cui_widget_append_window_command(widget, WindowRefresh, window);

	g_free(filename);
	return widget;
}

CuiWidget *
cui_action_area_new_for_node_numbers(
		CuiWindow   *window,
		Background  *background,
		CuiFilename *file,
		GtkTextMark *begin,
		GtkTextMark *end)
{
	GtkWidget *appbar;
	CuiWidget *widget;
	gchar *tooltip;
	gchar *filename;
	
	g_assert(background != NULL && file != NULL);
	g_assert(begin != NULL && end != NULL);
	appbar = lookup_widget(background->widget, "appbar1");
		
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_marks(widget, begin, end);

	filename = cui_filename_get_filename(file);
	tooltip = g_strdup_printf(
			_("The major and minor numbers of the device node. "
			  "Double click to show the properties for '%s'."),
					filename);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_append_file_command(widget, FileShowProperties, file);

	g_free(filename);
	return widget;
}

CuiWidget *
cui_action_area_new_for_owner(
		CuiWindow   *window,
		Background  *background,
		CuiFilename *file,
		GtkTextMark *begin,
		GtkTextMark *end)
{
	GtkWidget *appbar;
	CuiWidget *widget;
	gchar *tooltip;
	gchar *filename;
	
	g_assert(background != NULL && file != NULL);
	g_assert(begin != NULL && end != NULL);
	appbar = lookup_widget(background->widget, "appbar1");
		
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_marks(widget, begin, end);

	/*
	 * Syntax highlight and active highlight.
	 */
	cui_widget_set_active_style(widget, "username_active");
	cui_window_apply_tag(window, "username", begin, end);
		
	
	filename = cui_filename_get_filename(file);
	if (cui_filename_is_dir(file))
	tooltip = g_strdup_printf(
			_("The owner of the directory. "
			  "Right click for the user menu.")
		  );
	else
	tooltip = g_strdup_printf(
			_("The owner of the file. "
			  "Right click for the user menu.")
		  );
	cui_widget_set_tooltip(widget, tooltip);
	//cui_widget_append_file_command(widget, FileShowProperties, file);
	//if (!cui_filename_is_dir(file))
		cui_widget_append_window_command(widget, WindowRefresh, window);

	g_free(filename);
	return widget;
}

CuiWidget *
cui_action_area_new_for_link(
		CuiWindow   *window,
		Background  *background,
		CuiFilename *file,
		GtkTextMark *begin,
		GtkTextMark *end)
{
	GtkWidget *appbar;
	CuiWidget *widget;
	gchar *tooltip;
	gchar *filename;
	
	g_assert(background != NULL && file != NULL);
	g_assert(begin != NULL && end != NULL);
	appbar = lookup_widget(background->widget, "appbar1");
		
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_marks(widget, begin, end);
	cui_widget_set_active_style(widget, "normal_active");

	filename = cui_filename_get_filename(file);
	if (cui_filename_is_dir(file))
	tooltip = g_strdup_printf( _("The number of links to the directory. "));
	else
	tooltip = g_strdup_printf( _("The number of links to the file. "));
	cui_widget_set_tooltip(widget, tooltip);
	//cui_widget_append_file_command(widget, FileShowProperties, file);
	//if (!cui_filename_is_dir(file))
		cui_widget_append_window_command(widget, WindowRefresh, window);

	g_free(filename);
	return widget;
}

CuiWidget *
cui_action_area_new_for_group(
		CuiWindow   *window,
		Background  *background,
		CuiFilename *file,
		GtkTextMark *begin,
		GtkTextMark *end)
{
	GtkWidget *appbar;
	CuiWidget *widget;
	gchar *tooltip;
	gchar *filename;
	
	g_assert(background != NULL && file != NULL);
	g_assert(begin != NULL && end != NULL);
	appbar = lookup_widget(background->widget, "appbar1");
		
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_marks(widget, begin, end);

	/*
	 * Syntax highlight and active highlight.
	 */
	cui_widget_set_active_style(widget, "groupname_active");
	cui_window_apply_tag(window, "groupname", begin, end);

	filename = cui_filename_get_filename(file);
	if (cui_filename_is_dir(file))
	tooltip = g_strdup_printf( _("The group owner of the directory. "));
	else
	tooltip = g_strdup_printf( _("The group owner of the file. "));
	cui_widget_set_tooltip(widget, tooltip);
	//cui_widget_append_file_command(widget, FileShowProperties, file);
	//if (!cui_filename_is_dir(file))
		cui_widget_append_window_command(widget, WindowRefresh, window);

	g_free(filename);
	return widget;
}

CuiWidget *
cui_action_area_new_for_moddate(
		CuiWindow   *window,
		Background  *background,
		CuiFilename *file,
		GtkTextMark *begin,
		GtkTextMark *end)
{
	GtkWidget *appbar;
	CuiWidget *widget;
	gchar *tooltip;
	CuiFileType filetype;
	gchar *filename;
	
	g_assert(background != NULL && file != NULL);
	g_assert(begin != NULL && end != NULL);
	appbar = lookup_widget(background->widget, "appbar1");
		
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_marks(widget, begin, end);
	cui_widget_set_active_style(widget, "normal_active");

	filename = cui_filename_get_filename(file);
	filetype = cui_filename_get_filetype(file);
	tooltip = g_strdup_printf(
			_("Double click to refresh modification time "
					"for '%s'."),
					filename);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_append_file_command(widget, FileRefresh, file);
	if (!cui_filename_is_dir(file))
		cui_widget_append_window_command(widget, WindowRefresh, window);

	g_free(filename);
	return widget;
}
/**
 * cui_syntax_highlight_apply_filename:
 * @bg: the #Background
 * @file: the file which we make a syntax highlight for
 * @begin: the start point of the highlighted text
 * @end: the end point of the highlighted text
 * 
 * Creates a syntax highlight for the filename based on the type of the file,
 * the mode and the filename extension.
 */
void
cui_syntax_highlight_apply_filename(
		Background  *bg,
		CuiFilename *file,
		GtkTextMark *begin,
		GtkTextMark *end)
{
	GtkTextIter biter, eiter;
	gchar *tag = NULL;
	g_return_if_fail(bg != NULL && file != NULL);
	g_return_if_fail(begin != NULL && end != NULL);
	
	
	gtk_text_buffer_get_iter_at_mark(bg->text_buffer, &biter, begin);
	gtk_text_buffer_get_iter_at_mark(bg->text_buffer, &eiter, end);
	if (cui_filename_is_dir(file))
		tag = "directory";
	else if (cui_filename_is_user_execute(file) ||
			cui_filename_is_group_execute(file) ||
			cui_filename_is_others_execute(file) )
		tag = "executable";
	else if (cui_filename_is_compressed(file))
		tag = "archive";
		
		
	if (tag == NULL)
		return;
	gtk_text_buffer_apply_tag_by_name(bg->text_buffer, tag, &biter, &eiter);
}

/**
 * cui_action_area_new_for_mode_owner:
 * @window: a @CuiWindow containing the mode string
 * @background: a @Background where the action area goes
 * @file: a #CuiFile of this mode string
 * @begin: the start point of the string
 * @end: the end point of the string
 *
 * Creates some action areas for changing the mode of a file on the user rights
 * string which is a three character long string (eg. "rwx").
 */
void
cui_action_area_new_for_mode_owner(
		CuiWindow   *window,
		Background  *background,
		CuiFilename *file,
		GtkTextMark *begin,
		GtkTextMark *end)
{
	GtkTextIter iter, end_iter;
	gchar *text;
	GtkWidget *appbar;
	CuiWidget *widget;
	gchar *tooltip;
	gchar *filename;
	gchar *username;
	gchar *active_tag;
	
	/*
	 * Checking and collecting information.
	 */
	g_assert(background != NULL && file != NULL);
	g_assert(begin != NULL && end != NULL);
	appbar = lookup_widget(background->widget, "appbar1");
	filename = cui_filename_get_filename(file);
	username = cui_filename_get_owner(file);
	gtk_text_buffer_get_iter_at_mark(background->text_buffer, &iter, begin);
	/*
	 * First field: read right for the owner.
	 */
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_user_read(file)){
		tooltip = g_strdup_printf(
				_("Read file '%s' for user %s is granted. "
				"Double click to revoke it."),
				filename, username);
		cui_widget_append_file_command(widget, 
				FileRevokeReadUser, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
				_("Read file '%s' for user %s is forbidden. "
				"Double click to grant it."),
				filename, username);
		cui_widget_append_file_command(widget, FileGrantReadUser, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	/*
	 * Next field: write right for the owner.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_user_write(file)){
		tooltip = g_strdup_printf(
				_("Write file '%s' for user %s is granted. "
				"Double click to revoke it."),
				filename, username);
		cui_widget_append_file_command(widget, 
				FileRevokeWriteUser, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
				_("Write file '%s' for user %s is forbidden. "
				"Double click to grant it."),
				filename, username);
		cui_widget_append_file_command(widget, 
				FileGrantWriteUser, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	
	/*
	 * Next field: execute right for the owner.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_user_execute(file)){
		tooltip = g_strdup_printf(
				_("Execute file '%s' for user %s is granted. "
				"Double click to revoke it."),
				filename, username);
		cui_widget_append_file_command(widget, 
				FileRevokeExecuteUser, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
				_("Execute file '%s' for user %s is forbidden. "
				"Double click to grant it."),
				filename, username);
		cui_widget_append_file_command(widget, 
				FileGrantExecuteUser, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	
	/*
	 * Freeing all the allocated strings.
	 */
	g_free(filename);
	g_free(username);
}

/**
 * cui_action_area_new_for_mode_group:
 * @window: a @CuiWindow containing the mode string
 * @background: a @Background where the action area goes
 * @file: a #CuiFile of this mode string
 * @begin: the start point of the string
 * @end: the end point of the string
 *
 * Creates some action areas for changing the mode of a file on the group rights
 * string which is a three character long string (eg. "rwx").
 */
void
cui_action_area_new_for_mode_group(
		CuiWindow   *window,
		Background  *background,
		CuiFilename *file,
		GtkTextMark *begin,
		GtkTextMark *end)
{
	GtkTextIter iter, end_iter;
	gchar *text;
	GtkWidget *appbar;
	CuiWidget *widget;
	gchar *tooltip;
	gchar *filename;
	gchar *groupname;
	gchar *active_tag;
	
	/*
	 * Checking and collecting information.
	 */
	g_assert(background != NULL && file != NULL);
	g_assert(begin != NULL && end != NULL);
	appbar = lookup_widget(background->widget, "appbar1");
	filename = cui_filename_get_filename(file);
	groupname = cui_filename_get_group(file);
	gtk_text_buffer_get_iter_at_mark(background->text_buffer, &iter, begin);
	/*
	 * First field: read right for the group.
	 */
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_group_read(file)){
		tooltip = g_strdup_printf(
			_("Read file '%s' for members of %s is granted. "
			"Double click to revoke it."),
			filename, groupname);
		cui_widget_append_file_command(widget, 
				FileRevokeReadGroup, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
			_("Read file '%s' for members of %s is forbidden. "
			"Double click to grant it."),
			filename, groupname);
		cui_widget_append_file_command(widget, 
				FileGrantReadGroup, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	/*
	 * Next field: write right for the group.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_group_write(file)){
		tooltip = g_strdup_printf(
			_("Write file '%s' for members of %s is granted. "
			"Double click to revoke it."),
			filename, groupname);
		cui_widget_append_file_command(widget, 
				FileRevokeWriteGroup, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
			_("Write file '%s' for members of %s is forbidden. "
			"Double click to grant it."),
			filename, groupname);
		cui_widget_append_file_command(widget, 
				FileGrantWriteGroup, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	/*
	 * Next field: execute right for the group.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_group_execute(file)){
		tooltip = g_strdup_printf(
			_("Execute file '%s' for members of %s is granted. "
			"Double click to revoke it."),
			filename, groupname);
		cui_widget_append_file_command(widget, 
				FileRevokeExecuteGroup, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
			_("Execute file '%s' for members of %s is forbidden. "
			"Double click to grant it."),
			filename, groupname);
		cui_widget_append_file_command(widget, 
				FileGrantExecuteGroup, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);

	/*
	 * Freeing all the allocated strings.
	 */
	g_free(filename);
	g_free(groupname);

}

/**
 * cui_action_area_new_for_mode_others:
 * @window: a @CuiWindow containing the mode string
 * @background: a @Background where the action area goes
 * @file: a #CuiFile of this mode string
 * @begin: the start point of the string
 * @end: the end point of the string
 *
 * Creates some action areas for changing the mode of a file on the others 
 * rights string which is a three character long string (eg. "rwx").
 */
void
cui_action_area_new_for_mode_others(
		CuiWindow   *window,
		Background  *background,
		CuiFilename *file,
		GtkTextMark *begin,
		GtkTextMark *end)
{
	GtkTextIter iter, end_iter;
	gchar *text;
	GtkWidget *appbar;
	CuiWidget *widget;
	gchar *tooltip;
	gchar *filename;
	gchar *active_tag;
	
	/*
	 * Checking and collecting information.
	 */
	g_assert(background != NULL && file != NULL);
	g_assert(begin != NULL && end != NULL);
	appbar = lookup_widget(background->widget, "appbar1");
	filename = cui_filename_get_filename(file);
	gtk_text_buffer_get_iter_at_mark(background->text_buffer, &iter, begin);
	/*
	 * First field: read right for the others.
	 */
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_others_read(file)){
		tooltip = g_strdup_printf(
			_("Read file '%s' for the others is granted. "
			"Double click to revoke it."),
			filename);
		cui_widget_append_file_command(widget, 
				FileRevokeReadOthers, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
			_("Read file '%s' for the others is forbidden. "
			"Double click to grant it."),
			filename);
		cui_widget_append_file_command(widget, 
				FileGrantReadOthers, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);

	/*
	 * Next field: write right for the others.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_others_write(file)){
		tooltip = g_strdup_printf(
			_("Write file '%s' for the others is granted. "
			"Double click to revoke it."),
			filename);
		cui_widget_append_file_command(widget, 
				FileRevokeWriteOthers, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
			_("Write file '%s' for the others is forbidden. "
			"Double click to grant it."),
			filename);
		cui_widget_append_file_command(widget, 
				FileGrantWriteOthers, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	
	/*
	 * Next field: execute right for the others.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_others_execute(file)){
		tooltip = g_strdup_printf(
			_("Execute file '%s' for the others is granted. "
			"Double click to revoke it."),
			filename);
		cui_widget_append_file_command(widget, 
				FileRevokeExecuteOthers, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
			_("Execute file '%s' for the others is forbidden. "
			"Double click to grant it."),
			filename);
		cui_widget_append_file_command(widget, 
				FileGrantExecuteOthers, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	/*
	 * Freeing all the allocated strings.
	 */
	g_free(filename);
}

/**
 * cui_action_area_new_for_mode:
 * @window: a @CuiWindow containing the mode string
 * @background: a @Background where the action area goes
 * @file: a #CuiFile of this mode string
 * @begin: the start point of the string
 * @end: the end point of the string
 *
 * Creates some action areas for changing the mode of a file on the mode string
 * which is a ten cahracter long string with the file type (eg. "-rwxr-----").
 * This function also handles the + sign at the end of the mode string showing
 * that there is at least one ACL entry for the file (eg. -rwxrw-r--+).
 */
void
cui_action_area_new_for_mode(
		CuiWindow   *window,
		Background  *background,
		CuiFilename *file,
		GtkTextMark *begin,
		GtkTextMark *end)
{
	GtkTextIter iter, end_iter;
	gchar *text;
	GtkWidget *appbar;
	CuiWidget *widget;
	gchar *tooltip;
	gchar *filename;
	gchar *username;
	gchar *groupname;
	gchar *active_tag;
	
	/*
	 * Checking and collecting information.
	 */
	g_assert(background != NULL && file != NULL);
	g_assert(begin != NULL && end != NULL);
	appbar = lookup_widget(background->widget, "appbar1");
	filename = cui_filename_get_filename(file);
	username = cui_filename_get_owner(file);
	groupname = cui_filename_get_group(file);
	gtk_text_buffer_get_iter_at_mark(background->text_buffer, &iter, begin);
	/*
	 * First field: read right for the owner.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_user_read(file)){
		tooltip = g_strdup_printf(
				_("Read file '%s' for user %s is granted. "
				"Double click to revoke it."),
				filename, username);
		cui_widget_append_file_command(widget, 
				FileRevokeReadUser, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
				_("Read file '%s' for user %s is forbidden. "
				"Double click to grant it."),
				filename, username);
		cui_widget_append_file_command(widget, FileGrantReadUser, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	//gtk_text_buffer_apply_tag_by_name(background->text_buffer, tag, &begin, &end);
	/*
	 * Next field: write right for the owner.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_user_write(file)){
		tooltip = g_strdup_printf(
				_("Write file '%s' for user %s is granted. "
				"Double click to revoke it."),
				filename, username);
		cui_widget_append_file_command(widget, 
				FileRevokeWriteUser, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
				_("Write file '%s' for user %s is forbidden. "
				"Double click to grant it."),
				filename, username);
		cui_widget_append_file_command(widget, 
				FileGrantWriteUser, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	/*
	 * Next field: execute right for the owner.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_user_execute(file)){
		tooltip = g_strdup_printf(
				_("Execute file '%s' for user %s is granted. "
				"Double click to revoke it."),
				filename, username);
		cui_widget_append_file_command(widget, 
				FileRevokeExecuteUser, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
				_("Execute file '%s' for user %s is forbidden. "
				"Double click to grant it."),
				filename, username);
		cui_widget_append_file_command(widget, 
				FileGrantExecuteUser, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	/*
	 * Next field: read right for the group.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_group_read(file)){
		tooltip = g_strdup_printf(
			_("Read file '%s' for members of %s is granted. "
			"Double click to revoke it."),
			filename, groupname);
		cui_widget_append_file_command(widget, 
				FileRevokeReadGroup, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
			_("Read file '%s' for members of %s is forbidden. "
			"Double click to grant it."),
			filename, groupname);
		cui_widget_append_file_command(widget, 
				FileGrantReadGroup, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);

	/*
	 * Next field: write right for the group.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_group_write(file)){
		tooltip = g_strdup_printf(
			_("Write file '%s' for members of %s is granted. "
			"Double click to revoke it."),
			filename, groupname);
		cui_widget_append_file_command(widget, 
				FileRevokeWriteGroup, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
			_("Write file '%s' for members of %s is forbidden. "
			"Double click to grant it."),
			filename, groupname);
		cui_widget_append_file_command(widget, 
				FileGrantWriteGroup, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	
	/*
	 * Next field: execute right for the group.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_group_execute(file)){
		tooltip = g_strdup_printf(
			_("Execute file '%s' for members of %s is granted. "
			"Double click to revoke it."),
			filename, groupname);
		cui_widget_append_file_command(widget, 
				FileRevokeExecuteGroup, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
			_("Execute file '%s' for members of %s is forbidden. "
			"Double click to grant it."),
			filename, groupname);
		cui_widget_append_file_command(widget, 
				FileGrantExecuteGroup, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);

	/*
	 * Next field: read right for the others.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_others_read(file)){
		tooltip = g_strdup_printf(
			_("Read file '%s' for the others is granted. "
			"Double click to revoke it."),
			filename);
		cui_widget_append_file_command(widget, 
				FileRevokeReadOthers, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
			_("Read file '%s' for the others is forbidden. "
			"Double click to grant it."),
			filename);
		cui_widget_append_file_command(widget, 
				FileGrantReadOthers, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);

	/*
	 * Next field: write right for the others.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_others_write(file)){
		tooltip = g_strdup_printf(
			_("Write file '%s' for the others is granted. "
			"Double click to revoke it."),
			filename);
		cui_widget_append_file_command(widget, 
				FileRevokeWriteOthers, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
			_("Write file '%s' for the others is forbidden. "
			"Double click to grant it."),
			filename);
		cui_widget_append_file_command(widget, 
				FileGrantWriteOthers, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	
	/*
	 * Next field: execute right for the others.
	 */
	gtk_text_iter_forward_char(&iter);
	widget = cui_action_area_new();
	cui_widget_set_background(widget, background);
	cui_widget_set_appbar(widget, appbar);
	cui_widget_set_area_by_iter(widget, &iter);
	if (cui_filename_is_others_execute(file)){
		tooltip = g_strdup_printf(
			_("Execute file '%s' for the others is granted. "
			"Double click to revoke it."),
			filename);
		cui_widget_append_file_command(widget, 
				FileRevokeExecuteOthers, file);
		active_tag = "yes_active";
	}else{
		tooltip = g_strdup_printf(
			_("Execute file '%s' for the others is forbidden. "
			"Double click to grant it."),
			filename);
		cui_widget_append_file_command(widget, 
				FileGrantExecuteOthers, file);
		active_tag = "no_active";
	}
	cui_widget_append_window_command(widget, WindowRefresh, window);
	cui_widget_set_tooltip(widget, tooltip);
	cui_widget_set_active_style(widget, active_tag);
	/*
	 * The action area to view ACL items.
	 */
	gtk_text_iter_forward_char(&iter);
	end_iter = iter;
	gtk_text_iter_forward_char(&end_iter);
	text = gtk_text_buffer_get_text(background->text_buffer, 
			&iter, &end_iter, TRUE);
	if (text[0] == '+') {
		widget = cui_action_area_new();
		cui_widget_set_background(widget, background);
		cui_widget_set_appbar(widget, appbar);
		cui_widget_set_area_by_iter(widget, &iter);
		tooltip = g_strdup_printf(
				_("Entry '%s' has an ACL list attached. "
					"Double click to view it."),
				filename);
		cui_widget_set_tooltip(widget, tooltip);
		cui_widget_append_file_command(widget, FileViewACLList, file);
		cui_widget_set_active_style(widget, "normal_active");
	}
	g_free(text);

	/*
	 * Freeing all the allocated strings.
	 */
	g_free(filename);
	g_free(username);
	g_free(groupname);
}


/*
 * Some old stuff. FIXME: we have a better solution for this.
 */
void
ls_place_error_message(CuiWindow *window,
		char *token,
		GtkTextMark *mbegin,
		GtkTextMark *mend)
{
	GtkTextIter begin, end;
	
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, &begin, mbegin);
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, &end, mend);
	gtk_text_buffer_apply_tag_by_name(window->text_buffer, "FgRed", &begin, &end);
}

