/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: terminal.h,v 1.1 2006/02/24 17:29:40 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: terminal.h,v $
 * Revision 1.1  2006/02/24 17:29:40  pipas
 * Initial revision
 *
 */
#ifndef TERMINAL_H
#define TERMINAL_H

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>

#include "cuiterm.h"
#include "command_generator.h"
#include "background.h"

/****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/

/**
 * TerminalStatus:
 * @TermUninitialized: the first prompt did not came yet
 * @TermReady: the terminal is ready, we have a prompt
 * @TermSendingCommand: sending a generated command
 * @TermSendingUserEnter: the user hit the Enter key
 * @TermExecuting: the terminal is executing a command
 */
typedef enum {
	TermUninitialized,
	TermReady,
	TermSendingCommand,
	TermSendingUserEnter,
	TermExecuting
} TerminalStatus;

/**
 * Terminal:
 * @command: the command the user editing 
 * @wd: the name of the working directory
 * @username: data from the prompt
 * @prompt_length: the length of the actual prompt in characters
 * @text_buffer: the #GtkTextBuffer holding the text
 * @text_view: the #GtkTextView holding the terminal text
 * @background: the background for this terminal
 * @command_generator: the generator for this terminal
 * @parser_cursor: FIXME
 * @lexer_cursor: the point where the lexer reads the terminal
 * @parsed_chars: how many characters has been parsed
 * @parse_tree: the parse tree for the last BASH command
 * @cmd: FIXME
 * @hostname: the hostname part of the prompt
 * @directory: the directory part of the prompt
 * @executing: TRUE while the terminal is executing a command
 * @pending_commands: more commands to execute if ready
 * @shell_stream: the stream we use to communicate with the shell
 * @last_command_window: the last embedded window
 * @hold_shift: are user press the shift key (in keyboard)
 */
typedef struct _Terminal Terminal;
struct _Terminal {
	GtkTextBuffer *text_buffer;
	GtkTextView   *text_view;

	Background *background;
	CuiGenerator *command_generator;
	
	GtkTextMark *parser_cursor;
	GtkTextMark *lexer_cursor;
	gint parsed_chars;

	GString *command;   
	GNode   *parse_tree;
	GString *cmd;
	GString *wd;
	GString *username;  
	GString *hostname;
	GString *directory;
	gint prompt_length; 

	gboolean executing;
	GList *pending_commands;
	
	GIOChannel *shell_stream;
	struct CuiWidget  *last_command_window;
	TerminalStatus status;
	gint hold_shift;
	gboolean overwrite;
};


/***************************************************************************
 * Utility functions handling character terminal.                          *
 *                                                                         *
 **************************************************************************/
Terminal *terminal_new(GtkTextView *text_view);
void terminal_debug_print_status(Terminal *t);
void terminal_send_init_commands(Terminal *t);
void terminal_apply_tag(Terminal *t, gchar *tag,
		GtkTextMark *mbegin,
		GtkTextMark *mend);
void terminal_set_working_directory(Terminal *t, GString *wd);
void terminal_set_prompt(Terminal *t, GString *prompt);
gint terminal_send_command(Terminal *t, gchar *command, gboolean send);
int terminal_get_char(Terminal *t, char *buf, int max_size);
gint terminal_get_token_marks(Terminal *terminal,
		char *token,
		GtkTextMark **mbegin,
		GtkTextMark **mend);
const gchar *terminal_get_working_directory(Terminal *terminal);
int terminal_get_buffer_char(char *buf, int max_size);

/*
 * Terminal sequence handling functions.
 */
gint terminal_sequence_overwrite(Terminal *t);
gint terminal_sequence_insert(Terminal *t);

gint terminal_sequence_text_with_attributes(Terminal *t, 
		gchar *buffer, 
		gint *args, gint argn);
gint terminal_sequence_text(Terminal *t, gchar *buffer);
gint terminal_sequence_prompt(Terminal *t, gchar *buffer);
void terminal_sequence_erase(Terminal *t);
void terminal_sequence_backspace(Terminal *t);
void terminal_sequence_backspace_n(Terminal *t, gint n);
void terminal_sequence_delete(Terminal *t);
void terminal_sequence_delete_to_end(Terminal *t);
void terminal_sequence_delete_to_begin(Terminal *t);
void terminal_sequence_delete_line(Terminal *t);
void terminal_sequence_delete_n(Terminal *t, gint n);
void terminal_sequence_cursor_home(Terminal *t);
void terminal_sequence_cursor_xy(Terminal *t, gint x, gint y);
void terminal_sequence_cursor_up(Terminal *t);
void terminal_sequence_cursor_down(Terminal *t);
void terminal_sequence_cursor_down_n(Terminal *t, gint n);
void terminal_sequence_cursor_right(Terminal *t);
void terminal_sequence_cursor_right_n(Terminal *t, gint n);
void terminal_sequence_cursor_left(Terminal *t);
void terminal_sequence_cursor_left_n(Terminal *t, gint n);

void terminal_get_command(Terminal *t);
gint terminal_command_starting(Terminal *t);
gint terminal_command_ended(Terminal *t);
gboolean type_timeout_function(gpointer data);
/***************************************************************************
 * Callback functions handling character terminal and shell I/O.           *
 *                                                                         *
 **************************************************************************/
gboolean read_from_shell(GIOChannel * source, 
		GIOCondition condition, 
		gpointer data);

gboolean
on_terminal_key_press_event(GtkWidget  *widget, 
		GdkEventKey  *event, 
		gpointer     user_data);

void terminal_changed(Terminal *t);

#endif
